@@ -53,9 +53,6 @@ static void
 cb_clipboard_toggled               (GtkToggleButton    *tb,
                                     ScreenshotData     *sd);
 static void
-cb_zimagez_toggled                 (GtkToggleButton    *tb,
-                                    ScreenshotData     *sd);
-static void
 cb_imgur_toggled                   (GtkToggleButton    *tb,
                                     ScreenshotData     *sd);
 static void
@@ -187,12 +184,6 @@ static void cb_clipboard_toggled (GtkToggleButton *tb, ScreenshotData *sd)
 
 
 
-static void cb_zimagez_toggled (GtkToggleButton *tb, ScreenshotData *sd)
-{
-  if (gtk_toggle_button_get_active (tb))
-    sd->action = UPLOAD;
-}
-
 static void cb_imgur_toggled (GtkToggleButton *tb, ScreenshotData *sd)
 {
   if (gtk_toggle_button_get_active (tb))
@@ -298,12 +289,15 @@ static void cb_combo_active_item_changed (GtkWidget *box, ScreenshotData *sd)
   GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (box));
   GtkTreeIter iter;
   gchar *active_command = NULL;
+  GAppInfo *app_info = NULL;
 
   gtk_combo_box_get_active_iter (GTK_COMBO_BOX (box), &iter);
   gtk_tree_model_get (model, &iter, 2, &active_command, -1);
+  gtk_tree_model_get (model, &iter, 3, &app_info, -1);
 
   g_free (sd->app);
   sd->app = active_command;
+  sd->app_info = app_info;
 }
 
 
@@ -342,6 +336,7 @@ static void add_item (GAppInfo *app_info, GtkWidget *liststore)
                       0, pixbuf,
                       1, name,
                       2, command,
+                      3, g_app_info_dup (app_info),
                       -1);
 
   /* Free the stuff */
@@ -385,16 +380,19 @@ static void set_default_item (GtkWidget *combobox, ScreenshotData *sd)
     {
       gchar *command = NULL;
       gboolean found = FALSE;
+      GAppInfo *app_info;
 
       /* Loop until finding the appropirate item, if any */
       do
         {
           gtk_tree_model_get (model, &iter, 2, &command, -1);
+          gtk_tree_model_get (model, &iter, 3, &app_info, -1);
 
           if (g_str_equal (command, sd->app))
             {
               gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combobox),
                                              &iter);
+              sd->app_info = app_info;
 
               found = TRUE;
             }
@@ -409,12 +407,14 @@ static void set_default_item (GtkWidget *combobox, ScreenshotData *sd)
         {
           gtk_tree_model_get_iter_first (model , &iter);
           gtk_tree_model_get (model, &iter, 2, &command, -1);
+          gtk_tree_model_get (model, &iter, 3, &app_info, -1);
 
           gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combobox), &iter);
 
           g_free (sd->app);
 
           sd->app = command;
+          sd->app_info = app_info;
         }
     }
   else
@@ -422,6 +422,7 @@ static void set_default_item (GtkWidget *combobox, ScreenshotData *sd)
       g_free (sd->app);
 
       sd->app = g_strdup ("none");
+      sd->app_info = NULL;
     }
 }
 
@@ -430,26 +431,17 @@ static void set_default_item (GtkWidget *combobox, ScreenshotData *sd)
 static GdkPixbuf
 *screenshot_get_thumbnail (GdkPixbuf *screenshot)
 {
-  gint width, height, i;
-  GdkPixbuf *thumbnail;
+  gint w = gdk_pixbuf_get_width (screenshot);
+  gint h = gdk_pixbuf_get_height (screenshot);
+  gint width = THUMB_X_SIZE;
+  gint height = THUMB_Y_SIZE;
 
-  width = gdk_pixbuf_get_width (screenshot);
-  height = gdk_pixbuf_get_height (screenshot);
-
-  if (width > height)
-    i = width / THUMB_X_SIZE;
+  if (G_LIKELY (w >= h))
+    height = width * h / w;
   else
-    i = height / THUMB_Y_SIZE;
+    width = height * w / h;
 
-  if (i == 0)
-    return gdk_pixbuf_copy (screenshot);
-
-  thumbnail = gdk_pixbuf_scale_simple (screenshot,
-                                       width/i,
-                                       height/i,
-                                       GDK_INTERP_BILINEAR);
-
-  return thumbnail;
+  return gdk_pixbuf_scale_simple (screenshot, width, height, GDK_INTERP_BILINEAR);
 }
 
 
@@ -923,7 +915,6 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   GtkWidget *actions_label, *actions_alignment, *actions_grid;
   GtkWidget *save_radio_button;
   GtkWidget *clipboard_radio_button, *open_with_radio_button;
-  GtkWidget *zimagez_radio_button;
   GtkWidget *imgur_radio_button;
 
   GtkListStore *liststore;
@@ -1005,7 +996,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   /* Save option radio button */
   save_radio_button = gtk_radio_button_new_with_mnemonic (NULL, _("Save"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (save_radio_button),
-                                (sd->action == SAVE));
+                                (sd->action & SAVE));
   g_signal_connect (G_OBJECT (save_radio_button), "toggled",
                     G_CALLBACK (cb_save_toggled), sd);
   gtk_widget_set_tooltip_text (save_radio_button, _("Save the screenshot to a PNG file"));
@@ -1022,7 +1013,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
                                    _("Copy the screenshot to the clipboard so that it can be "
                                      "pasted later"));
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (clipboard_radio_button),
-                                    (sd->action == CLIPBOARD));
+                                    (sd->action & CLIPBOARD));
       g_signal_connect (G_OBJECT (clipboard_radio_button), "toggled",
                         G_CALLBACK (cb_clipboard_toggled), sd);
       gtk_grid_attach (GTK_GRID (actions_grid), clipboard_radio_button, 0, 1, 1, 1);
@@ -1033,7 +1024,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
     gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (save_radio_button),
                                                  _("Open with:"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (open_with_radio_button),
-                                (sd->action == OPEN));
+                                (sd->action & OPEN));
   g_signal_connect (G_OBJECT (open_with_radio_button), "toggled",
                     G_CALLBACK (cb_open_toggled), sd);
   gtk_widget_set_tooltip_text (open_with_radio_button,
@@ -1041,7 +1032,7 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   gtk_grid_attach (GTK_GRID (actions_grid), open_with_radio_button, 0, 2, 1, 1);
 
   /* Open with combobox */
-  liststore = gtk_list_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
+  liststore = gtk_list_store_new (4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_APP_INFO);
   combobox = gtk_combo_box_new_with_model (GTK_TREE_MODEL (liststore));
   renderer = gtk_cell_renderer_text_new ();
   renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
@@ -1063,25 +1054,12 @@ GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd)
   /* Run the callback functions to grey/ungrey the correct widgets */
   cb_toggle_set_sensi (GTK_TOGGLE_BUTTON (open_with_radio_button), combobox);
 
-  /* Upload to zimagez radio button */
-  zimagez_radio_button =
-    gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (save_radio_button),
-                                                 _("Host on ZimageZ"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (zimagez_radio_button),
-                                (sd->action == UPLOAD));
-  gtk_widget_set_tooltip_text (zimagez_radio_button,
-                               _("Host the screenshot on ZimageZ, a free online "
-                                 "image hosting service"));
-  g_signal_connect (G_OBJECT (zimagez_radio_button), "toggled",
-                    G_CALLBACK (cb_zimagez_toggled), sd);
-  gtk_grid_attach (GTK_GRID (actions_grid), zimagez_radio_button, 0, 3, 1, 1);
-
   /* Upload to imgur radio button */
   imgur_radio_button =
     gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (save_radio_button),
                                                  _("Host on Imgur"));
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (imgur_radio_button),
-                                (sd->action == UPLOAD_IMGUR));
+                                (sd->action & UPLOAD_IMGUR));
   gtk_widget_set_tooltip_text (imgur_radio_button,
                                _("Host the screenshot on Imgur, a free online "
                                  "image hosting service"));
@@ -1184,14 +1162,11 @@ gchar
         /* Create the preview and the thumbnail */
         preview_ebox = gtk_event_box_new ();
         preview = gtk_image_new ();
+        gtk_widget_set_margin_end (preview, 12);
         gtk_container_add (GTK_CONTAINER (preview_ebox), preview);
         gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (chooser), preview_ebox);
 
-        thumbnail =
-          gdk_pixbuf_scale_simple (screenshot,
-                                   gdk_pixbuf_get_width(screenshot)/5,
-                                   gdk_pixbuf_get_height(screenshot)/5,
-                                   GDK_INTERP_BILINEAR);
+        thumbnail = screenshot_get_thumbnail (screenshot);
 
         gtk_image_set_from_pixbuf (GTK_IMAGE (preview), thumbnail);
         g_object_unref (thumbnail);
