@@ -112,6 +112,7 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   sd->screenshot_dir = screenshot_dir;
   sd->title = title;
   sd->app = app;
+  sd->app_info = NULL;
   sd->last_user = last_user;
 }
 
@@ -153,12 +154,16 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
 /* Opens the screenshot using application.
 @screenshot_path: the path to the saved screenshot.
 @application: the command to run the application.
+@app_info: GAppInfo object associated with application.
 */
 void
-screenshooter_open_screenshot (const gchar *screenshot_path, const gchar *application)
+screenshooter_open_screenshot (const gchar *screenshot_path, const gchar *application, GAppInfo *const app_info)
 {
-  gchar *command;
-  GError *error = NULL;
+  gpointer screenshot_file = NULL;
+  gboolean success = TRUE;
+  gchar   *command;
+  GError  *error = NULL;
+  GList   *files = NULL;
 
   g_return_if_fail (screenshot_path != NULL);
 
@@ -169,22 +174,31 @@ screenshooter_open_screenshot (const gchar *screenshot_path, const gchar *applic
 
   TRACE ("Application was not none");
 
-  command = g_strconcat (application, " ",
-                         "\"", screenshot_path, "\"", NULL);
+  if (app_info != NULL)
+    {
+      TRACE ("Launch the app");
 
-  TRACE ("Launch the command");
+      screenshot_file = g_file_new_for_path (screenshot_path);
+      files = g_list_append (NULL, screenshot_file);
+      success = g_app_info_launch (app_info, files, NULL, &error);
+      g_list_free_full (files, g_object_unref);
+    }
+  else if (application != NULL)
+    {
+      TRACE ("Launch the command");
 
-  /* Execute the command and show an error dialog if there was
-  * an error. */
-  if (!g_spawn_command_line_async (command, &error))
+      command = g_strconcat (application, " ", "\"", screenshot_path, "\"", NULL);
+      success = g_spawn_command_line_async (command, &error);
+      g_free (command);
+    }
+
+  /* report any error */
+  if (!success && error != NULL)
     {
       TRACE ("An error occured");
-
       screenshooter_error (_("<b>The application could not be launched.</b>\n%s"), error->message);
       g_error_free (error);
     }
-
-  g_free (command);
 }
 
 
