@@ -33,11 +33,10 @@ gboolean window = FALSE;
 gboolean region = FALSE;
 gboolean fullscreen = FALSE;
 gboolean mouse = FALSE;
-gboolean upload = FALSE;
 gboolean clipboard = FALSE;
 gboolean upload_imgur = FALSE;
-gchar *screenshot_dir;
-gchar *application;
+gchar *screenshot_dir = NULL;
+gchar *application = NULL;
 gint delay = 0;
 
 
@@ -82,11 +81,6 @@ static GOptionEntry entries[] =
     N_("Directory where the screenshot will be saved"),
     NULL
   },
-  {
-    "upload", 'u', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &upload,
-    N_("Host the screenshot on ZimageZ, a free online image hosting service"),
-    NULL
-  },
   {
     "imgur", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &upload_imgur,
     N_("Host the screenshot on Imgur, a free online image hosting service"),
@@ -150,6 +144,8 @@ int main (int argc, char **argv)
 
   ScreenshotData *sd = g_new0 (ScreenshotData, 1);
   sd->plugin = FALSE;
+  sd->app_info = NULL;
+  sd->action = 0;
 
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
@@ -194,53 +190,13 @@ int main (int argc, char **argv)
     }
 
   /* Exit if two actions options were given */
-  if (upload && (application != NULL))
-    {
-      g_printerr (conflict_error, "upload", "open");
-
-      g_free (sd);
-      return EXIT_FAILURE;
-    }
-  else if (upload && (screenshot_dir != NULL))
-    {
-      g_printerr (conflict_error, "upload", "save");
-
-      g_free (sd);
-      return EXIT_FAILURE;
-    }
-  else if (upload && clipboard)
-    {
-      g_printerr (conflict_error, "upload", "clipboard");
-
-      g_free (sd);
-      return EXIT_FAILURE;
-    }
-  else if ((application != NULL) && (screenshot_dir != NULL))
+  if ((application != NULL) && (screenshot_dir != NULL))
     {
       g_printerr (conflict_error, "open", "save");
 
       g_free (sd);
       return EXIT_FAILURE;
     }
-  else if ((application != NULL) && clipboard)
-    {
-      g_printerr (conflict_error, "open", "clipboard");
-
-      g_free (sd);
-      return EXIT_FAILURE;
-    }
-  else if ((screenshot_dir != NULL) && clipboard)
-    {
-      g_printerr (conflict_error, "save", "clipboard");
-
-      g_free (sd);
-      return EXIT_FAILURE;
-    }
-  else if (upload_imgur && upload)
-    {
-      g_printerr (conflict_error, "upload", "imgur");
-      return EXIT_FAILURE;
-    }
   else if (upload_imgur && (screenshot_dir != NULL))
     {
       g_printerr (conflict_error, "imgur", "save");
@@ -260,8 +216,6 @@ int main (int argc, char **argv)
     g_printerr (ignore_error, "save");
   if (upload_imgur && !(fullscreen || window || region))
     g_printerr (ignore_error, "imgur");
-  if (upload && !(fullscreen || window || region))
-    g_printerr (ignore_error, "upload");
   if (clipboard && !(fullscreen || window || region))
     g_printerr (ignore_error, "clipboard");
   if (delay && !(fullscreen || window || region))
@@ -319,30 +273,26 @@ int main (int argc, char **argv)
           sd->action = OPEN;
           sd->action_specified = TRUE;
         }
-      else if (upload)
-        {
-          sd->app = g_strdup ("none");
-          sd->action = UPLOAD;
-          sd->action_specified = TRUE;
-        }
-      else if (clipboard)
-        {
-          sd->app = g_strdup ("none");
-          sd->action = CLIPBOARD;
-          sd->action_specified = TRUE;
-        }
       else if (upload_imgur)
         {
-          sd->app = g_strdup ("none");
           sd->action = UPLOAD_IMGUR;
           sd->action_specified = TRUE;
         }
-      else
+      else if (screenshot_dir != NULL)
         {
-          sd->app = g_strdup ("none");
           sd->action = SAVE;
+          sd->action_specified = TRUE;
         }
 
+      if (clipboard)
+        {
+          sd->action |= CLIPBOARD;
+          sd->action_specified = TRUE;
+        }
+
+      if (!sd->app)
+        sd->app = g_strdup ("none");
+
       /* If the user gave a directory name, check that it is valid */
       if (screenshot_dir != NULL)
         {
